Package: skewr
Title: Visualize Intensities Produced by Illumina's Human Methylation
        450k BeadChip
Version: 1.44.0
Authors@R: c(person("Ryan", "Putney", role = c("cre", "aut"), email =
        "ryanputney@gmail.com"), person("Steven", "Eschrich", role =
        "aut"), person("Anders", "Berglund", role = "aut"))
Description: The skewr package is a tool for visualizing the output of
        the Illumina Human Methylation 450k BeadChip to aid in quality
        control. It creates a panel of nine plots. Six of the plots
        represent the density of either the methylated intensity or the
        unmethylated intensity given by one of three subsets of the
        485,577 total probes. These subsets include Type I-red, Type
        I-green, and Type II.The remaining three distributions give the
        density of the Beta-values for these same three subsets. Each
        of the nine plots optionally displays the distributions of the
        "rs" SNP probes and the probes associated with imprinted genes
        as series of 'tick' marks located above the x-axis.
Depends: R (>= 3.1.1), methylumi, wateRmelon, mixsmsn,
        IlluminaHumanMethylation450kmanifest
Imports: minfi, S4Vectors (>= 0.19.1), RColorBrewer
Suggests: GEOquery, knitr, minfiData
VignetteBuilder: knitr
License: GPL-2
biocViews: DNAMethylation, TwoChannel, Preprocessing, QualityControl
Author: Ryan Putney [cre, aut], Steven Eschrich [aut], Anders Berglund
        [aut]
Maintainer: Ryan Putney <ryanputney@gmail.com>
git_url: https://git.bioconductor.org/packages/skewr
git_branch: RELEASE_3_23
git_last_commit: 33814ce
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:47:49 UTC; biocbuild

#exportPattern("^[^\\.]")

export('getSNparams')
export('panelPlots')
export('getMethyLumiSet')
export('preprocess')
export('subsetProbes')
export('getBarcodes')
#export('plotComponents')

importFrom('mixsmsn', 'smsn.mix', 'mix.hist')
importFrom('methylumi', 'methylumIDAT', 'normalizeMethyLumiSet',
           'methylumi.bgcorr', 'controlData', 'negctls', 'betas')
importFrom('minfi', 'getProbeInfo')
import('IlluminaHumanMethylation450kmanifest')
importFrom('wateRmelon', 'dasen', 'swan', 'iDMR')
import('RColorBrewer')
import('S4Vectors')

#################################################
##
## Functions for getting MethyLumiSet objects
## and for preprocessing of them
##
#################################################

getBarcodes <- function(path = getwd(), recurse = FALSE){
  unique <- vector(mode = 'character')
  for(i in 1:length(path)){
    unique <- c(unique, grep('_Red.idat$', list.files(path[i], recursive = recurse), value = TRUE))
    unique <- gsub('_Red.idat$', '', unique)
  }

  unique
}


getMethyLumiSet <- function(path = getwd(), barcodes = NULL,
                            norm = c("none", "illumina", "SWAN", "dasen"), bg.corr = TRUE){
  norm <- match.arg(norm)
  if(is.null(barcodes)) barcodes <- getBarcodes(path)
  object <- methylumIDAT(barcodes = barcodes, idatPath = path)
  if(norm != "none") object <- preprocess(object, norm = norm, bg.corr = bg.corr)
  object
}

preprocess <- function(MethyLumiSet,
                       norm = c("none", "illumina", "SWAN", "dasen"), bg.corr = TRUE){

  norm <- match.arg(norm)
  if(norm == "none"){
    if(bg.corr == TRUE){
      object <- methylumi.bgcorr(MethyLumiSet, method = 'illumina', controls = negctls(controlData(MethyLumiSet)))
    }
  }
  if(norm == "illumina"){
    object <- normalizeMethyLumiSet(MethyLumiSet)
    if(bg.corr == TRUE){
      object <- methylumi.bgcorr(object, method = 'illumina', controls = negctls(controlData(object)))
    }
  }
  else if(norm == "SWAN"){
    object <- swan(MethyLumiSet, return.MethylSet = TRUE)
  }
  else if(norm == "dasen"){
    object <- dasen(MethyLumiSet, ret2 = TRUE)
  }
  object
}

subsetProbes <- function(object, allele = c('M', 'U'), type = c('I-red', 'I-green', 'II'),
                         cg = TRUE, snps = TRUE, idmr = TRUE, ch = FALSE){
  data(IlluminaHumanMethylation450kmanifest, envir = environment())
  allele <- match.arg(allele)
  type <- match.arg(type)

  if(cg){
    I.red.names <- minfi::getProbeInfo(IlluminaHumanMethylation450kmanifest, "I-Red")$Name
    I.grn.names <- minfi::getProbeInfo(IlluminaHumanMethylation450kmanifest, "I-Green")$Name
    II.names <- minfi::getProbeInfo(IlluminaHumanMethylation450kmanifest, "II")$Name
  }
  else I.red.names <- I.grn.names <- II.names <- NULL

  if(snps){
    snpI.red.names <- S4Vectors::subset(getProbeInfo(IlluminaHumanMethylation450kmanifest, "SnpI"), Color == "Red")$Name
    snpI.grn.names <- S4Vectors::subset(getProbeInfo(IlluminaHumanMethylation450kmanifest, "SnpI"), Color == "Grn")$Name
    snpII.names <- minfi::getProbeInfo(IlluminaHumanMethylation450kmanifest, "SnpII")$Name
    #print(subset(getProbeInfo(IlluminaHumanMethylation450kmanifest,'SnpII'), Color=='Red'))
    #snpI.grn.names <- snpI.red.names <- snpII.names <- NULL
  }
  else snpI.red.names <- snpI.grn.names <- snpII.names <- NULL

  if(!idmr){
    data(iDMR, envir = environment())
    I.red.names <- I.red.names[!I.red.names %in% iDMR]
    I.grn.names <- I.grn.names[!I.grn.names %in% iDMR]
    II.names <- II.names[!II.names %in% iDMR]
  }

  if(!ch){
    II.names <- II.names[!grepl('^ch', II.names)]
  }

  if(is(object, "MethylSet")){
    if(allele == 'M') mat <- minfi::getMeth(object)
    else mat <- minfi::getUnmeth(object)

    if(type == 'I-red') mat <- mat[rownames(mat) %in% c(I.red.names, snpI.red.names),,drop=FALSE]
    else if(type == 'I-green') mat <- mat[rownames(mat) %in% c(I.grn.names, snpI.grn.names),,drop=FALSE]
    else mat <- mat[rownames(mat) %in% c(II.names, snpII.names),,drop=FALSE]
  }
  else if(is(object, "MethyLumiSet")){
    if(allele == 'M') mat <- methylumi::methylated(object)
    else mat <- methylumi::unmethylated(object)

    if(type == 'I-red') mat <- mat[rownames(mat) %in% c(I.red.names, snpI.red.names),,drop=FALSE]
    else if(type == 'I-green') mat <- mat[rownames(mat) %in% c(I.grn.names, snpI.grn.names),,drop=FALSE]
    else mat <- mat[rownames(mat) %in% c(II.names, snpII.names),,drop=FALSE]
  }

  mat
}


prepSamp <- function(sample){
  if(is.null(sample)) sample <- NULL
  else{
    sample[sample < 0] <- 0
    sample <- sample + 1
    sample <- log2(sample)
  }
  sample
}

dSN <- function(y, mu = 0, sigma2 = 1, shape=1){
  dens <- 2*dnorm(y, mu, sqrt(sigma2))*pnorm(shape*((y - mu)/sqrt(sigma2)))
  return(dens)
}

d.mixedSN <- function(x, pi1, mu, sigma2, shape){
    g <- length(pi1)
    dens <- 0
    for (j in 1:g) dens <- dens + pi1[j]*dSN(x, mu[j], sigma2[j], shape[j])
    return(dens)
  }

SNmean <- function(mu, sigma2, shape){

   xi <- mu
   omega <- sqrt(sigma2)
   alpha <- shape

   C <- sqrt(2 / pi)
   delta <- alpha / sqrt(1 + alpha^2)

   mean <- xi + omega * delta * C
   mean
}

SNsd <- function(mu, sigma2, shape){

	xi <- mu
	omega2 <- sigma2
	alpha <- shape

	delta <- alpha / sqrt(1 + alpha^2)

	c <- (2 * delta^2) / pi

	variance <- omega2 * (1 - c)

	sd <- sqrt(variance)

}

plotMix <- function(sample, model){

	sample <- prepSamp(sample)

	mix.hist(sample, model, ylim = c(0, .5), xlim = c(0, 18))

}



plotComponents <- function(sample, model, idmrVec = NULL, snpsVec = NULL, chip = "", channel = "", type = "", legend = FALSE){

	naVec <- NULL
	if(legend) sub <- type else sub <- NULL
	#if(is.matrix(sample) || !is(model, 'Skew.normal')) stop('Function only accepts a single sample and a single model')

	x <- seq(0, 18, length = 1000000)
	pii <- model$pii
	mu <- model$mu
	sigma2 <- model$sigma2
	shape <- model$shape
	modes <- model$modes
	means <- model$means
	dens.list <- model$dens.list
	order <- order(mu)
	mu <- mu[order]
	pii <- pii[order]
	sigma2 <- sigma2[order]
	shape <- shape[order]
	modes <- modes[order]
	means <- means[order]
	dens.list <- dens.list[order]
	g <- length(pii)
	myColors <- brewer.pal(9, "Set1")[1:g]
	tickColors <- brewer.pal(9, "Set1")[4:6]
	name <- colnames(sample)

	dens.mix <- d.mixedSN(x, pii, mu, sigma2, shape) #xx to x
	y.max <- max(dens.mix)+0.2   #########################
	x.y.max <- x[which(dens.mix == y.max)]



	hist(sample, breaks = 60,probability=TRUE, border = 'grey', ylim = c(0, y.max),
		xlim = c(4,16),xlab = paste(channel, "Log2 Intensity"), main = NULL, sub = sub)
      	lines(x, dens.mix, lwd = 1) #xx to x


## finding the mean and dropping the line from the curve to the x-axis
	for(i in 1:g){
		lines(x, pii[i] * dens.list[[i]]$y, type = 'l', col = myColors[i], lwd = 1)
#		means[i] <- SNmean(mu[i], sigma2[i], shape[i])
		dens.max <- which.max(dens.list[[i]]$y)
#		modes[i] <- dens.list[[i]]$x[dens.max[i]]
		lines(segments(means[i], 0, means[i], pii[i] * dSN(means[i], mu[i], sigma2[i], shape[i]), lty = 2, lwd = 1, col = myColors[i]))
		lines(segments(modes[i], 0, modes[i], pii[i] * dens.list[[i]]$y[dens.max], col = myColors[i], lwd = 1))
	}
	lines(density(sample, na.rm = TRUE), lty = 3, lwd = 1)
	means <- round(means, 3)
	modes <- round(modes, 3)


## plot segments for points in pointsVec
	if(!is.null(idmrVec) && !is.null(snpsVec) && !is.null(naVec)){
		for(i in 1:length(naVec)){
			lines(segments(naVec[i], y.max * 0.10, naVec[i], y.max * 0.15, col = tickColors[3]))
		}
		for(i in 1:length(idmrVec)){
			lines(segments(idmrVec[i], y.max * .050, idmrVec[i], y.max * .099, col = tickColors[2]))
		}
		for(i in 1:length(snpsVec)){
			lines(segments(snpsVec[i], 0, snpsVec[i], y.max * .049, col = tickColors[1]))
		}
	}
	else if(!is.null(idmrVec) && !is.null(snpsVec)){
		for(i in 1:length(idmrVec)){
			lines(segments(idmrVec[i], y.max * .050, idmrVec[i], y.max * .099, col = tickColors[2]))
		}
		for(i in 1:length(snpsVec)){
			lines(segments(snpsVec[i], 0, snpsVec[i], y.max * .049, col = tickColors[1]))
		}
	}
	else if(!is.null(idmrVec) && !is.null(naVec)){
		for(i in 1:length(naVec)){
			lines(segments(naVec[i], y.max * .050, naVec[i], y.max * .099, col = tickColors[3]))
		}
		for(i in 1:length(idmrVec)){
			lines(segments(idmrVec[i], 0, idmrVec[i], y.max * .049, col = tickColors[2]))
		}
	}
	else if(!is.null(snpsVec) && !is.null(naVec)){
		for(i in 1:length(naVec)){
			lines(segments(naVec[i], y.max * .050, naVec[i], y.max * .099, col = tickColors[3]))
		}
		for(i in 1:length(snpsVec)){
			lines(segments(snpsVec[i], 0, snpsVec[i], y.max * .049, col = tickColors[1]))
		}
	}
	else if(!is.null(idmrVec)){
		for(i in 1:length(idmrVec)){
			lines(segments(idmrVec[i], 0, idmrVec[i], y.max * .05, col = tickColors[2]))
		}
	}
	else if(!is.null(snpsVec)){
		for(i in 1:length(snpsVec)){
			lines(segments(snpsVec[i], 0, snpsVec[i], y.max * .05, col = tickColors[1]))
		}
	}
	else if(!is.null(naVec)){
		for(i in 1:length(naVec)){
			lines(segments(naVec[i], 0, naVec[i], y.max * .05, col = tickColors[3]))
		}
	}
## create legend
## to alternate means and modes:
##
##	as.vector(rbind(means, modes))
	#if(x.y.max < 10) x.coord <- 4 #13
	#else x.coord <- 4 #5
  x.coord <- 4
	y.coord <- y.max #+ .05
	if(legend){
		legend(x.coord, y.coord, c(paste(rep(c("mean =", "mode ="), each = g), c(means, modes)), "dens sum","non-par dens") , col = c(rep(myColors,2), 'black', 'black'),
        		lty = c(rep(c(2, 1), each = g), 1, 3), lwd = c(rep(2, 2*g+1), 1), xpd = NA, cex = 0.9) #TRUE)
	}
}


panelPlots <- function(MethyLumiSet, typeIRedModels, typeIGreenModels, typeIIModels, 
                       plot = c('panel', 'frames'), samp.num = NULL, frame.nums = 1:9, norm = '',
					   idmr = TRUE, snps = TRUE){
	if(!any(grepl('^rs', rownames(MethyLumiSet)))) snps <- FALSE
	plot <- match.arg(plot)

	if (plot == 'frames'){
		if(is.null(samp.num)){
			if(ncol(MethyLumiSet) > 1) stop('To see single plots, a sample number must be specified')
			start <- 1
			end <- 1
			legend <- TRUE
		}
		else{
			if(length(samp.num) > 1) stop('Only one sample number can be specified at this time')
			start <- samp.num
			end <- samp.num
			legend <- TRUE
		}
		
	}
	else{
		if(!is.null(samp.num)){
			if(length(samp.num) > 1) stop('Only one sample number can be specified at this time')
			start <- samp.num
			end <- samp.num
			legend <- FALSE
		}
		else{
			start <- 1
			end <- ncol(MethyLumiSet)
			legend <- FALSE
		}
	}
	frame.nums <- 1:9 %in% frame.nums

	tickColors <- brewer.pal(9, "Set1")[4:6]
	all.betas <- betas(MethyLumiSet)
	typeI.red.meth <- prepSamp(subsetProbes(MethyLumiSet, 'M', 'I-red', idmr = idmr, snps = snps))
	typeI.red.unmeth <- prepSamp(subsetProbes(MethyLumiSet, 'U', 'I-red', idmr = idmr, snps = snps))
	typeI.red.betas <- cbind(all.betas[rownames(typeI.red.meth),], NA)
	typeI.green.meth <- prepSamp(subsetProbes(MethyLumiSet, 'M', 'I-green', idmr = idmr, snps = snps))
	typeI.green.unmeth <- prepSamp(subsetProbes(MethyLumiSet, 'U', 'I-green', idmr = idmr, snps = snps))
	typeI.green.betas <- cbind(all.betas[rownames(typeI.green.meth),], NA)
	typeII.meth <- prepSamp(subsetProbes(MethyLumiSet, 'M', 'II', idmr = idmr, snps = snps))
	typeII.unmeth <- prepSamp(subsetProbes(MethyLumiSet, 'U', 'II', idmr = idmr, snps = snps))
	typeII.betas <- cbind(all.betas[rownames(typeII.meth),], NA)

	if(snps){
		snp.names <- grep('rs', rownames(MethyLumiSet), value = TRUE)
		snpI.red.meth <- cbind(typeI.red.meth[rownames(typeI.red.meth) %in% snp.names,], NA)
		snpI.red.unmeth <- cbind(typeI.red.unmeth[rownames(typeI.red.unmeth) %in% snp.names,], NA)
		snpI.red.betas <- cbind(all.betas[rownames(snpI.red.meth),], NA)
		snpI.green.meth <- cbind(typeI.green.meth[rownames(typeI.green.meth) %in% snp.names,], NA)
		snpI.green.unmeth <- cbind(typeI.green.unmeth[rownames(typeI.green.unmeth) %in% snp.names,], NA)
		snpI.green.betas <- cbind(all.betas[rownames(snpI.green.meth),], NA)
		snpII.meth <- cbind(typeII.meth[rownames(typeII.meth) %in% snp.names,], NA)
		snpII.unmeth <- cbind(typeII.meth[rownames(typeII.unmeth) %in% snp.names,], NA)
		snpII.betas <- cbind(all.betas[rownames(snpII.meth),], NA)
	}
	if(idmr){
		data(iDMR, envir = environment())
		iDMR.typeI.red.meth <- cbind(typeI.red.meth[rownames(typeI.red.meth) %in% iDMR,], NA)
		iDMR.typeI.red.unmeth <- cbind(typeI.red.unmeth[rownames(typeI.red.unmeth) %in% iDMR,], NA)
		iDMR.typeI.red.betas <- cbind(all.betas[rownames(iDMR.typeI.red.meth),], NA)
		iDMR.typeI.green.meth <- cbind(typeI.green.meth[rownames(typeI.green.meth) %in% iDMR,], NA)
		iDMR.typeI.green.unmeth <- cbind(typeI.green.unmeth[rownames(typeI.green.unmeth) %in% iDMR,], NA)
		iDMR.typeI.green.betas <- cbind(all.betas[rownames(iDMR.typeI.green.meth),], NA)
		iDMR.typeII.meth <- cbind(typeII.meth[rownames(typeII.meth) %in% iDMR,], NA)
		iDMR.typeII.unmeth <- cbind(typeII.unmeth[rownames(typeII.unmeth) %in% iDMR,], NA)
		iDMR.typeII.betas <- cbind(all.betas[rownames(iDMR.typeII.meth),], NA)
	}


	finishPlot <- function(){
	  mtext(paste(colnames(MethyLumiSet)[i], norm), 3, adj = 0.5, line = 2, cex = 1.2, outer = TRUE)
	  oldPar <- par(fig = c(0, 1, 0, 1), oma = c(0, 0, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
	  plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
	  legend("bottom", c("SNP", "iDMR"), xpd = TRUE, horiz = TRUE, inset = c(0, 0),
	         bty = "n", pch = 15, col = tickColors[1:2], cex = 1.2)
    par(oldPar)
	}

	for(i in start:end){
		if (plot == 'panel') par(mfcol = c(3,3), mgp = c(2, 1, 0), mar=c(4,4,0.5,0.5), oma = c(4,0,4,0))
		else par(oma = c(4,0,4,0), mar=c(4,4,2,2), mgp = c(2,1,0))

		if(frame.nums[1]){
			plotComponents(typeI.red.meth[,i], typeIRedModels[[1]][[i]], if(idmr) iDMR.typeI.red.meth[,i] else NULL, if(snps) snpI.red.meth[,i] else NULL, NULL, "Methylated", "I-red", legend)
			if (plot == 'frames') finishPlot()
		}
		if(frame.nums[2]){ 
			plotComponents(typeI.red.unmeth[,i], typeIRedModels[[2]][[i]], if(idmr) iDMR.typeI.red.unmeth[,i] else NULL, if(snps) snpI.red.unmeth[,i] else NULL, NULL, "Unmethylated", "I-red", legend)
			if (plot == 'frames') finishPlot()
		}
		if(frame.nums[3]){
			if (plot == 'frames') sub <- "Type I-Red" else sub <- NULL
			plot(density(typeI.red.betas[,i], na.rm = TRUE), xaxt = 'n', bty = 'n', main = NA, xlab = "Beta values", sub = sub)
			axis(1, at = c(0.0, 0.2, 0.4, 0.6, 0.8, 1.0), labels = c('0.0', '0.2', '0.4', '0.6', '0.8', '1.0'))
			axis(1, at = 0.5, labels = FALSE, col.ticks = 'red', tck = -.08)
			h <- max(density(typeI.red.betas[, i], na.rm = TRUE)$y)
			if(idmr && snps){
				idmrBetas <- iDMR.typeI.red.betas[,i]
				idmrBetas <- na.omit(idmrBetas)
				snpBetas <- snpI.red.betas[,i]
				snpBetas <- na.omit(snpBetas)
				idmrHgt <- h * .1
				idmrBtm <- h * .051
				for(j in 1:length(idmrBetas)){
					lines(segments(idmrBetas[j], idmrBtm , idmrBetas[j], idmrHgt, col = tickColors[2]))
				}
				snpHgt <- h * .049
				for(j in 1:length(snpBetas)){
					lines(segments(snpBetas[j], 0, snpBetas[j], snpHgt, col = tickColors[1]))
				}
			}
			else if(idmr && !snps){
				idmrBetas <- iDMR.typeI.red.betas[,i]
				idmrBetas <- na.omit(idmrBetas)
				height <- h * .05
				for(j in 1:length(idmrBetas)){
					lines(segments(idmrBetas[j], 0, idmrBetas[j], height, col = tickColors[2]))
				}
			}
			else if(!idmr && snps){
				snpBetas <- snpI.red.betas[,i]
				snpBetas <- na.omit(snpBetas)
				height <- h * .05
				for(j in 1:length(snpBetas)){
					lines(segments(snpBetas[j], 0, snpBetas[j], height, col = tickColors[1]))
				}
			}
			if (plot == 'frames') finishPlot()
		}
		if(frame.nums[4]){
			plotComponents(typeI.green.meth[,i], typeIGreenModels[[1]][[i]], if(idmr) iDMR.typeI.green.meth[,i] else NULL, if(snps) snpI.green.meth[,i] else NULL, NULL, "Methylated", "I-green", legend)
			if (plot == 'frames') finishPlot()
		}
		if(frame.nums[5]){
			plotComponents(typeI.green.unmeth[,i], typeIGreenModels[[2]][[i]], if(idmr) iDMR.typeI.green.unmeth[,i] else NULL, if(snps) snpI.green.unmeth[,i] else NULL, NULL, "Unmethylated", "I-green", legend)
			if (plot == 'frames') finishPlot()
		}
		if(frame.nums[6]){
			if (plot == 'frames') sub <- "Type I-Green" else sub <- NULL
			plot(density(typeI.green.betas[,i], na.rm = TRUE), xaxt = 'n', bty = 'n', main = NA, xlab = "Beta values", sub = sub)
			axis(1, at = c(0.0, 0.2, 0.4, 0.6, 0.8, 1.0), labels = c('0.0', '0.2', '0.4', '0.6', '0.8', '1.0'))
			axis(1, at = 0.5, labels = FALSE, col.ticks = 'red', tck = -.08)
			h <- max(density(typeI.green.betas[,i], na.rm = TRUE)$y)
			if(idmr && snps){
				idmrBetas <- iDMR.typeI.green.betas[,i]
				idmrBetas <- na.omit(idmrBetas)
				snpBetas <- snpI.green.betas[,i]
				snpBetas <- na.omit(snpBetas)
				idmrHgt <- h * .1
				idmrBtm <- h * .051
				for(j in 1:length(idmrBetas)){
					lines(segments(idmrBetas[j], idmrBtm , idmrBetas[j], idmrHgt, col = tickColors[2]))
				}
				snpHgt <- h * .049
				for(j in 1:length(snpBetas)){
					lines(segments(snpBetas[j], 0, snpBetas[j], snpHgt, col = tickColors[1]))
				}
			}
			else if(idmr && !snps){
				idmrBetas <- iDMR.typeI.green.betas[,i]
				idmrBetas <- na.omit(idmrBetas)
				height <- h * .05
				for(j in 1:length(idmrBetas)){
					lines(segments(idmrBetas[j], 0, idmrBetas[j], height, col = tickColors[2]))
				}
			}
			else if(!idmr && snps){
				snpBetas <- snpI.green.betas[,i]
				snpBetas <- na.omit(snpBetas)
				height <- h * .05
				for(j in 1:length(snpBetas)){
					lines(segments(snpBetas[j], 0, snpBetas[j], height, col = tickColors[1]))
				}
			}
			if (plot == 'frames') finishPlot()
		}
		if(frame.nums[7]){
			plotComponents(typeII.meth[,i], typeIIModels[[1]][[i]], if(idmr) iDMR.typeII.meth[,i] else NULL, if(snps) snpII.meth[,i] else NULL, NULL, "Methylated (green)", "II", legend)
			if (plot == 'frames') finishPlot()
		}
		if(frame.nums[8]){
			plotComponents(typeII.unmeth[,i], typeIIModels[[2]][[i]], if(idmr) iDMR.typeII.unmeth[,i] else NULL, if(snps) snpII.unmeth[,i] else NULL, NULL, "Unmethylated (red)", "II", legend)
			if (plot == 'frames') finishPlot()
		}
		if(frame.nums[9]){
			if (plot == 'frames') sub <- "Type II" else sub <- NULL
			plot(density(typeII.betas[,i], na.rm = TRUE), xaxt = 'n', bty = 'n', main = NA, xlab = "Beta values", sub = sub)
			axis(1, at = c(0.0, 0.2, 0.4, 0.6, 0.8, 1.0), labels = c('0.0', '0.2', '0.4', '0.6', '0.8', '1.0'))
			axis(1, at = 0.5, labels = FALSE, col.ticks = 'red', tck = -.08)
			h <- max(density(typeII.betas[,i], na.rm = TRUE)$y)
			if(idmr && snps){
				idmrBetas <- iDMR.typeII.betas[,i]
				idmrBetas <- na.omit(idmrBetas)
				snpBetas <- snpII.betas[,i]
				snpBetas <- na.omit(snpBetas)
				idmrHgt <- h * .1
				idmrBtm <- h * .051
				for(j in 1:length(idmrBetas)){
					lines(segments(idmrBetas[j], idmrBtm , idmrBetas[j], idmrHgt, col = tickColors[2]))
				}
				snpHgt <- h * .049
				for(j in 1:length(snpBetas)){
					lines(segments(snpBetas[j], 0, snpBetas[j], snpHgt, col = tickColors[1]))
				}
			}
			else if(idmr && !snps){
				idmrBetas <- iDMR.typeII.betas[,i]
				idmrBetas <- na.omit(idmrBetas)
				height <- h * .05
				for(j in 1:length(idmrBetas)){
					lines(segments(idmrBetas[j], 0, idmrBetas[j], height, col = tickColors[2]))
				}
			}
			else if(!idmr && snps){
				snpBetas <- snpII.betas[,i]
				snpBetas <- na.omit(snpBetas)
				height <- h * .05
				for(j in 1:length(snpBetas)){
					lines(segments(snpBetas[j], 0, snpBetas[j], height, col = tickColors[1]))
				}
			}
			if (plot == 'frames') finishPlot()
		}
	
		if(plot == 'panel'){
			mtext("Type I-Red", 3, adj = 0.15, outer = TRUE)
			mtext("Type I-Green", 3, adj = 0.5, outer = TRUE)
			mtext("Type II", 3, adj = 0.85, outer = TRUE)
			finishPlot()
		}

	}

}

############################################################
##
## All-in-one function that allows for subsetting
## of probes and skew normal modeling.
## Returns a Skew.normal object as produced by
## the smsn.mix function from the mixsmsn package, but which
## includes the means, modes, and densities of the
## individual components
##
############################################################



getSNparams <- function(MethyLumiSet, allele = c('M', 'U'),
                        type = c('I-red', 'I-green', 'II'), snps = TRUE, idmr = TRUE, ch = FALSE){

	object <- subsetProbes(MethyLumiSet, allele = allele, type = type, snps = snps, idmr = idmr, ch = ch)
	object <- prepSamp(object)

	if (type == 'II') g <- 2
	else g <- 3

	x <- seq(0, 18, length = 1000000)
	dens <- matrix(NA_real_, ncol = g, nrow = length(x))
	dens.list <- vector("list", g)
	means <- vector("numeric", g)
	modes <- vector("numeric", g)
	dens.max <- vector("numeric", g)

	if(is.matrix(object)){
		n.samp <- ncol(object)
		mix <- apply(object, 2, function(xx){
  											xx <- na.omit(xx)
												smsn.mix(xx, nu = 3, g = g, obs.prob = TRUE, calc.im = FALSE)
												})

		for(i in 1:ncol(object)){
			pii <- mix[[i]]$pii
			mu <- mix[[i]]$mu
			sigma2 <- mix[[i]]$sigma2
			shape <- mix[[i]]$shape
			for(j in 1:g){
				dens[,j] <- dSN(x, mu[j], sigma2[j], shape[j])
				dens.list[[j]] <- data.frame(x = x, y = dens[,j])
				means[j] <- SNmean(mu[j], sigma2[j], shape[j])
				dens.max[j] <- which.max(dens.list[[j]]$y)
				modes[j] <- dens.list[[j]]$x[dens.max[j]]
			}
			mix[[i]]$means <- means
			mix[[i]]$modes <- modes
			mix[[i]]$dens.list <- dens.list
		}
	}
	else{
		mix <- smsn.mix(na.omit(object), nu = 3, g = g, obs.prob = TRUE, calc.im  = FALSE)
		pii <- mix$pii
		mu <- mix$mu
		sigma2 <- mix$sigma2
		shape <- mix$shape
		for(j in 1:g){
			dens[,j] <- dSN(x, mu[j], sigma2[j], shape[j])
			dens.list[[j]] <- data.frame(x = x, y = dens[,j])
			means[j] <- SNmean(mu[j], sigma2[j], shape[j])
			dens.max[j] <- which.max(dens.list[[j]]$y)
			modes[j] <- dens.list[[j]]$x[dens.max[j]]
		}
		mix$means <- means
		mix$modes <- modes
		mix$dens.list <- dens.list
	}
	mix

}

## ----knitr, include=FALSE, echo=FALSE-----------------------------------------
library(knitr)
opts_chunk$set(tidy=FALSE,size='scriptsize', cache=FALSE, cache.comments=FALSE)

## ----options,eval=TRUE,echo=FALSE,results='hide'--------------------
options(width=70)

## ----install, eval=FALSE--------------------------------------------
# packages.install('mixsmsn')
# if (!requireNamespace("BiocManager", quietly=TRUE))
#     install.packages("BiocManager")
# BiocManager::install(c('skewr', 'methylumi', 'minfi', 'wateRmelon',
#            'IlluminaHumanMethylation450kmanifest', 'IRanges'))

## ----minfiData, eval=FALSE------------------------------------------
# BiocManager::install('minfiData')

## ----load, message=FALSE, warning=FALSE-----------------------------
library(skewr)
library(minfiData)

## ----getIDATs, eval=TRUE, warning=FALSE, message=FALSE--------------
baseDir <- system.file("extdata/5723646052", package = "minfiData")
barcodes <- getBarcodes(path = baseDir)
barcodes

## ----getMethyLumi, eval=FALSE, warning=FALSE, message=FALSE---------
# methylumiset.raw <- getMethyLumiSet(path = baseDir, barcodes =  barcodes[1:2])
# methylumiset.illumina <- preprocess(methylumiset.raw, norm = 'illumina',
#                                     bg.corr = FALSE)

## ----getMelon, eval=TRUE--------------------------------------------
data(melon)
melon.raw <- melon[,11]

## ----getMethyLumiNormEx, eval=FALSE---------------------------------
# melon.illumina <- preprocess(melon.raw, norm = 'illumina', bg.corr = TRUE)
# melon.SWAN <- preprocess(melon.raw, norm = 'SWAN')
# melon.dasen <- preprocess(melon.raw, norm = 'dasen')

## ----mixes, eval=TRUE-----------------------------------------------
sn.raw.meth.I.red <- getSNparams(melon.raw, 'M', 'I-red')
sn.raw.unmeth.I.red <- getSNparams(melon.raw, 'U', 'I-red')
sn.raw.meth.I.green <- getSNparams(melon.raw, 'M', 'I-green')
sn.raw.unmeth.I.green <- getSNparams(melon.raw, 'U', 'I-green')
sn.raw.meth.II <- getSNparams(melon.raw, 'M', 'II')
sn.raw.unmeth.II <- getSNparams(melon.raw, 'U', 'II')

## ----mixesdasen, eval=FALSE-----------------------------------------
# sn.dasen.meth.I.red <- getSNparams(melon.dasen, 'M', 'I-red')
# sn.dasen.unmeth.I.red <- getSNparams(melon.dasen, 'U', 'I-red')
# sn.dasen.meth.I.green <- getSNparams(melon.dasen, 'M', 'I-green')
# sn.dasen.unmeth.I.green <- getSNparams(melon.dasen, 'U', 'I-green')
# sn.dasen.meth.II <- getSNparams(melon.dasen, 'M', 'II')
# sn.dasen.unmeth.II <- getSNparams(melon.dasen, 'U', 'II')

## ----list, eval=TRUE------------------------------------------------
raw.I.red.mixes <- list(sn.raw.meth.I.red, sn.raw.unmeth.I.red)
raw.I.green.mixes <- list(sn.raw.meth.I.green, sn.raw.unmeth.I.green)
raw.II.mixes <- list(sn.raw.meth.II, sn.raw.unmeth.II)

## ----panelPlot1, eval=TRUE, echo=TRUE, dev='png', dpi=300, fig.cap='Panel Plot for One Sample'----
panelPlots(melon.raw, raw.I.red.mixes, raw.I.green.mixes,
           raw.II.mixes, norm = 'Raw')

## ----panelPlot2, eval=FALSE, echo=TRUE, dev='png', dpi=600----------
# panelPlots(melon.dasen,
#            list(sn.dasen.meth.I.red, sn.dasen.unmeth.I.red),
#            list(sn.dasen.meth.I.green, sn.dasen.unmeth.I.green),
#            list(sn.dasen.meth.II, sn.dasen.unmeth.II), norm = 'dasen')

## ----listAccess, eval=TRUE------------------------------------------
class(sn.raw.meth.I.red[[1]])
names(sn.raw.meth.I.red[[1]])
sn.raw.meth.I.red[[1]]$means
sn.raw.meth.I.red[[1]]$modes

## ----singlePlots, eval=TRUE, echo=TRUE, dev='png', dpi=300, fig.cap=c('Single Frame Showing Skew-normal Components', 'Single Frame Showing Beta Distribution for Type I Red')----
panelPlots(melon.raw, raw.I.red.mixes, raw.I.green.mixes,
           raw.II.mixes, plot='frames', frame.nums=c(1,3), norm='Raw')

## ----sessionInfo, results='asis', echo=FALSE------------------------
toLatex(sessionInfo())

%\VignetteIndexEntry{An Introduction to the skewr Package}
%\VignetteEngine{knitr::knitr}
%\VignetteDepends{skewr}
%\VignetteDepends{IlluminaHumanMethylation450kmanifest}
%\VignetteDepends{methylumi}
%\VignetteDepends{minfi}
%\vignetteDepends{minfiData}
%\VignetteDepends{wateRmelon}

%\documentclass[draft]{article}
\documentclass[12pt]{article}

\usepackage{graphicx}
\graphicspath{{./figure/}}
\DeclareGraphicsExtensions{.pdf, .png, .jpeg}
\usepackage{amsmath}
%\usepackage{color,hyperref}
\usepackage{fullpage}
\usepackage[numbers]{natbib}
\PassOptionsToPackage{hyphens}{url}\usepackage{color, hyperref}
\definecolor{darkblue}{rgb}{0.0,0.0,0.75}
\hypersetup{colorlinks,breaklinks,
            linkcolor=darkblue,urlcolor=darkblue,
            anchorcolor=darkblue,citecolor=darkblue}
\usepackage{parskip}
\usepackage{microtype}

\newcommand{\Rcode}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{\textsf{#1}}
\newcommand{\software}[1]{\textsf{#1}}
\newcommand{\R}{\software{R}}

\title{An Introduction to the \textbf{skewr} Package}
\author{Ryan Putney \and Steven Eschrich \and Anders Berglund}
\date{\today}


\begin{document}


<<knitr, include=FALSE, echo=FALSE>>=
library(knitr)
opts_chunk$set(tidy=FALSE,size='scriptsize', cache=FALSE, cache.comments=FALSE)
@

<<options,eval=TRUE,echo=FALSE,results='hide' >>=
options(width=70)
@
\maketitle
\setcounter{secnumdepth}{1}

\section{Introduction}

The \Rpackage{skewr} package is a tool for visualizing the output of the Illumina Human Methylation 450k BeadChip(450k) to aid in quality control. It creates a panel of nine plots. Six of the plots represent the density of either the methylated intensity or the unmethylated intensity given by one of three subsets of the 485,577 total probes. These subsets include Type I-red, Type I-green, and Type II. The remaining three distributions give the density of the $\beta$-values for these same three subsets. Each of the nine plots optionally displays the distributions of the \textsf{"rs"} SNP probes and the probes associated with imprinted genes\citep{Pidsley:2013} as a series of 'tick' marks located above the x-axis.



\subsection{Importance of the data}

DNA methylation is an epigenetic modification of the genome believed to play a role in gene expression. Hypermethylation and hypomethylation have the potential to either silence or 'turn on' specific genes, respectively. For this reason, the role that methylation plays in disease processes, such as cancer, is of particular interest to researchers. The introduction of 450k which can efficiently query the methylation status of more than 450,000 CpG sites has given rise to the need of being able to analyze the generated data in an equally efficient, as well as accurate, manner.

\subsection{450k Design}

The 450k utilizes two separate assay methods on a single chip. Approximately 135,000 of the total number of probes on the 450k are of the Infinium Type I technology, as used on the preceding 27k BeadChip. The Type I probes have both a methylated and an unmethylated probe type for each CpG site. The final base of these probe types are designed to match either the methylated \textbf{C} or the \textbf{T} which results from bisulfite conversion of an unmethylated \textbf{C}. A single base extension results in the addition of a colored dye. The color of the fluorescent dye used to measure the intensities of these probes is the same for both the methylated and unmethylated types. The Type II probes utilize one probe for both the methylated and unmethylated CpG locus. The single base extension step itself determines the methylation status of the locus. If the interrogated locus has a methylated \textbf{C}, then a green-labeled \textbf{G} is added, while a red-labeled \textbf{A} is added if the locus has the converted \textbf{T} denoting an unmethylated locus. For this reason, the level of methylation for the type II probes is always measured in the green channel, while level of unmethylation in the red. These factors mean that there are potentially six subsets of the total signal intensity: Type I-red methylated, Type I-red unmethylated, Type I-green methylated, Type I-green unmethylated, Type II methylated, and Type II unmethylated.

The preferred metric for evaluating the level of methylation for a given probe is usually the $\beta$-value, which is calculated as a ratio of the methylated signal intensity over the sum of the methylated and unmethylated signal intensities and a small offset value $\alpha$, which is usually 100:
\begin{align*}
  \beta = \frac{M}{M + U + \alpha}
\end{align*}

The $\beta$-value has the advantages of being the Illumina recommended metric and of being natural and straightforward\citep{Du:2010}.

Differences in the performance of the Type I and Type II probes\citep{Bibikova:2011,Dedeurwaerder:2011} and the existence of dye bias introduced by the two-color design\citep{Dedeurwaerder:2013}, however, have been observed. Much work has been done to correct these confounding factors, but their efficacy is usually judged in $\beta$ space.

\subsection{Proposed Use of \Rpackage{skewr}}

\Rpackage{skewr} is designed to visualize the array data in $\log_2$ intensity space. By analyzing the data in this way, we believe that a more accurate understanding of the biological variation may be attained.

As a step in that direction, we found that the $\log_2$ distributions of the intensities may be modeled as a mixture of skew-normal distributions. A three-component model fits the Type I intensity distributions well, while two components generally fit the Type II distributions the best. We have observed a difficulty in fitting a Skew-normal mixture model to the Type II intensity distributions, especially the unmethylated probes. We have verified, however, that the two-component model works very well for samples that carry a reasonable assumption of purity.

The location of the individual components themselves may give insight into the true signal-to-noise ratio, as well as possible mechanisms of non-specific binding. The posterior probabilities for the intensities of individual probes may prove useful in distinguishing the true biological variability in the methylation levels.

\subsection{Finite Mixture of Skew-normal Distributions}

\Rpackage{skewr} utilizes the \Rpackage{mixsmsn} package to estimate the parameters for the Skew-normal components that make up the finite mixture model for the intensity distributions\citep{Prates:2013}. \Rpackage{mixsmsn} deals with the family of distributions known as the scale mixtures of the skew-normal distributions(SMSN)\citep{Basso:2010}.

If a random variable $Z$ has a skew-normal distribution with location parameter $\mu$, scale parameter $\sigma^2$, and skewness, or shape, parameter $\lambda$, its density is given by:

\begin{align} \label{eq:1}
\psi(z) = 2\phi(z; \mu, \sigma^2)\Phi\left(\frac{\lambda(z-\mu)}{\sigma}\right),
\end{align}

where $\phi(\cdot ; \mu, \sigma^2)$ is the probability density function and $\Phi(\cdot)$ is the cumulative distribution function, both of the univariate normal distribution. Random variable $Z$ is then denoted as $Z\sim\mathrm{SN}(\mu, \sigma^2, \lambda)$.

$Y$ is a random variable with an SMSN distribution if:

\begin{align}
Y = \mu + U^{-1/2}Z,
\end{align}

where $\mu$ is the location parameter, $Z \sim \mathrm{SN}(0, \sigma^2, \lambda)$, and $U$ is a positive random variable given by the distribution function $H(\cdot, \nu)$. Then $U$ becomes the scale factor, and its distribution $H(\cdot, \nu)$ is the mixing distribution indexed by the parameter $\nu$ which may be univariate or multivariate. Since \Rpackage{skewr} is only dealing with the skew-normal distribution of the SMSN family $U$ always has the value $1$, and the parameter $\nu$ has no significance. Therefore,

\begin{align}
Y = \mu + Z, \quad Z \sim \mathrm{SN}(0, \sigma^2, \lambda)
\end{align}

A finite mixture of skew-normal distributions for a random sample $\mathbf{y} = (y_1, y_2, \dots, Y_n)$ with $g$ number of components is given by:

\begin{align}
f(y_i, \Theta)=\sum_{j=1}^{g}p_j\psi(y_i;\theta_j),\quad p_j \geq 0, \quad \sum_{j=1}^{g}p_j=1, \quad i=1, \dots, n, \quad j=1, \dots, g,
\end{align}

where $\theta_j = (\mu_j, \sigma^2_j, \lambda_j)$ are the parameters for the $j$\textsuperscript{th} skew-normal component, $p_1, \dots, p_g$ are the mixing probabilities, or weights, for the components, and $\Theta$ is a vector of all the parameters: $\Theta = ((p_1,\dots,p_g), \theta_1,\dots,\theta_g)$.

Finally, the estimated posterior probability $\hat{z}_{ij}$ is given by:

\begin{align}
\hat{z}_{ij} = \frac{\hat{p}_j\mathrm{SN}(y_i; \hat{\mu}_j,\hat{\sigma}^2_j, \hat{\lambda}_j)}{\sum_{k=1}^{g}\hat{p}_k\mathrm{SN}(y_i; \hat{\mu}_k,\hat{\sigma}^2_k, \hat{\lambda}_k)}, \quad i=1,\dots,n, \quad j=1,\dots,g,
\end{align}

which is essentially the probability of finding a given point in the $j$\textsuperscript{th} component over the probability of finding it in the mixture model.

\section{Getting Started}
\subsection{Installation}
\Rpackage{skewr} is a package that is designed to work with several preexisting packages. Therefore, a fair number of dependencies are required. The following packages must be installed:

<<install, eval=FALSE>>=
packages.install('mixsmsn')
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install(c('skewr', 'methylumi', 'minfi', 'wateRmelon',
           'IlluminaHumanMethylation450kmanifest', 'IRanges'))
@
And to run this vignette as written:

<<minfiData, eval=FALSE>>=
BiocManager::install('minfiData')
@

\subsection{Load \Rpackage{skewr}}
<<load, message=FALSE, warning=FALSE>>=
library(skewr)
library(minfiData)
@

\clearpage
\section{Sample Session}

\Rpackage{skewr} provides a convenience function for retrieving clean barcode names from all idat files found in the path, or vector of paths, given as a parameter. \Rcode{getMethyLumiSet} is a wrapper function utilizing the \Rcode{methylumIDAT} function provided by \Rpackage{methylumi}\citep{methylumi}. \Rcode{getMethyLumiSet} will process all idat files in the path to the directory given by the first argument, or default to the working directory if none is given. A vector of barcodes may be provided if only those specific idat files are to be processed. The default output will be a raw MethyLumiSet, unless a normalization method is specified when calling \Rcode{getMethyLumiSet}. It is unlikely that preprocessing will be desired when calling \Rcode{getMethyLumiSet} unless one is only interested in a single normalization method. It is much more likely that \Rcode{getMethyLumiSet} will be called to assign the raw \Rcode{MethyLumiSet} object to a label. Then the \Rcode{preprocess} method provided by the \Rpackage{skewr} may be called to return a number of objects with different normalization methods applied\citep{Maksimovic:2012,Pidsley:2013}.

As an example of the use of the \Rcode{getBarcodes} function provided by \Rpackage{skewr}, we will start by retrieving the barcodes of some idat files provided by \Rpackage{minfiData}\citep{minfiData}.

<<getIDATs, eval=TRUE, warning=FALSE, message=FALSE>>=
baseDir <- system.file("extdata/5723646052", package = "minfiData")
barcodes <- getBarcodes(path = baseDir)
barcodes
@

<<getMethyLumi, eval=FALSE, warning=FALSE, message=FALSE>>=
methylumiset.raw <- getMethyLumiSet(path = baseDir, barcodes =  barcodes[1:2])
methylumiset.illumina <- preprocess(methylumiset.raw, norm = 'illumina',
                                    bg.corr = FALSE)
@

To allow for a more efficient demonstration, however, the rest of the vignette will utilize a \Rcode{MethyLumiSet} object that is supplied by the Bioconductor package \Rpackage{wateRmelon}. This reduced data set only contains 3363 features out of the 485,577 total probes. We will only use one sample, but \Rpackage{skewr} will plot panels for all samples contained within the \Rcode{MethyLumiSet} passed to it.

<<getMelon, eval=TRUE>>=
data(melon)
melon.raw <- melon[,11]
@

Additional normalization methods may be performed as follows:

<<getMethyLumiNormEx, eval=FALSE>>=
melon.illumina <- preprocess(melon.raw, norm = 'illumina', bg.corr = TRUE)
melon.SWAN <- preprocess(melon.raw, norm = 'SWAN')
melon.dasen <- preprocess(melon.raw, norm = 'dasen')
@
\clearpage
\Rcode{getSNparams} will subset the probes by either methylated, \texttt{M}, or  unmethylated, \texttt{U}, and \texttt{I-red}, \texttt{I-green}, or \texttt{II}. The subset of probes are then fitted to a skew-normal finite mixture model. The mixture modelling is provided by the \Rcode{smsn.mix} method of the \Rpackage{mixsmsn} package. Assign the return value or each of the six \Rcode{getSNparams} calls to a separate label.

<<mixes, eval=TRUE>>=
sn.raw.meth.I.red <- getSNparams(melon.raw, 'M', 'I-red')
sn.raw.unmeth.I.red <- getSNparams(melon.raw, 'U', 'I-red')
sn.raw.meth.I.green <- getSNparams(melon.raw, 'M', 'I-green')
sn.raw.unmeth.I.green <- getSNparams(melon.raw, 'U', 'I-green')
sn.raw.meth.II <- getSNparams(melon.raw, 'M', 'II')
sn.raw.unmeth.II <- getSNparams(melon.raw, 'U', 'II')
@

If you want to compare the raw data for your experiment with the same data after a normalization method has been performed, you would carry out the same steps for each of your normalized \Rcode{MethyLumiSet}s. For example:

<<mixesdasen, eval=FALSE>>=
sn.dasen.meth.I.red <- getSNparams(melon.dasen, 'M', 'I-red')
sn.dasen.unmeth.I.red <- getSNparams(melon.dasen, 'U', 'I-red')
sn.dasen.meth.I.green <- getSNparams(melon.dasen, 'M', 'I-green')
sn.dasen.unmeth.I.green <- getSNparams(melon.dasen, 'U', 'I-green')
sn.dasen.meth.II <- getSNparams(melon.dasen, 'M', 'II')
sn.dasen.unmeth.II <- getSNparams(melon.dasen, 'U', 'II')
@

Before the panel plots can be made, the values returned by \Rcode{getSNparams} must be put into a list so that probes with the same assay and channel are in a separate list object with the methylated probes listed first followed by the unmethylated. Note that \Rpackage{skewr} is designed to create a series of panel plots for an entire experiment. It will not work if one attempts to index a single sample and its accompanying skew-normal models. See \autoref{sec:single} for information on how to better view a single sample within an experiment.

<<list, eval=TRUE>>=
raw.I.red.mixes <- list(sn.raw.meth.I.red, sn.raw.unmeth.I.red)
raw.I.green.mixes <- list(sn.raw.meth.I.green, sn.raw.unmeth.I.green)
raw.II.mixes <- list(sn.raw.meth.II, sn.raw.unmeth.II)
@

The \Rcode{panelPlots} method takes the original \Rcode{MethyLumiSet} object as the first parameter. The following parameters consist of the listed \texttt{I-red}, \texttt{I-green}, and \texttt{II} models, in that order.

<<panelPlot1, eval=TRUE, echo=TRUE, dev='png', dpi=300, fig.cap='Panel Plot for One Sample'>>=
panelPlots(melon.raw, raw.I.red.mixes, raw.I.green.mixes,
           raw.II.mixes, norm = 'Raw')
@

The \Rcode{samp.num} parameter of \Rcode{panelPlots}, may also be specified if plots for only one sample out of an experimenter is wanted. The \Rcode{samp.num} is an integer indexing the sample column within the \Rcode{MethyLumiSet} object.

Of course, the listing of the Skew-normal objects may be done in the call to \Rcode{panelPlots}.

<<panelPlot2, eval=FALSE, echo=TRUE, dev='png', dpi=600>>=
panelPlots(melon.dasen,
           list(sn.dasen.meth.I.red, sn.dasen.unmeth.I.red),
           list(sn.dasen.meth.I.green, sn.dasen.unmeth.I.green),
           list(sn.dasen.meth.II, sn.dasen.unmeth.II), norm = 'dasen')
@

\Rcode{panelPlots} will produce a panel plot for each sample in your experiment, \autoref{fig:panelPlot1}. Producing panel plots of the same experiment with different types of normalization applied may allow a better understanding of what the normalization method is actually doing to the data. %\autoref{fig:panel2}.

%\begin{figure}[ht!]
%\centering
%\includegraphics[width=\linewidth]{panelPlot1-1.png}
%\caption{Panel Plot for One Sample}
%\label{fig:panel1}
%\end{figure}

%\begin{figure}[ht!]
%\centering
%\includegraphics[width=\linewidth]{panelPlot2-1.png}
%\caption{Panel Plot for One Sample after dasen}
%\label{fig:panel2}
%\end{figure}

\clearpage
\section{Single Plots}
\label{sec:single}

It is possible to call \Rcode{panelPlots} with the \Rcode{plot} parameter as \Rcode{plot = 'frames'}. In this case, a sample number for a single sample contained within your experiment must also be passed as a parameter. If \Rcode{panelPlots} is called in this manner, nine separate, large plots will be created. Each of these nine plots correspond to a single pane that would be found in the panel plot for the selected sample. In addition, each of these separate plots, except for the beta plots, will contain a legend with useful information, such as the means and modes for each of the components of the mixture. The means and modes are also contained as part of the \Rcode{Skew.normal} object returned by \Rcode{getSNparams} and may be accessed in the following manner:

<<listAccess, eval=TRUE>>=
class(sn.raw.meth.I.red[[1]])
names(sn.raw.meth.I.red[[1]])
sn.raw.meth.I.red[[1]]$means
sn.raw.meth.I.red[[1]]$modes
@

To generate each panel as a single plot, see examples~\autoref{fig:singlePlots1}~and~\autoref{fig:singlePlots2}:

<<singlePlots, eval=TRUE, echo=TRUE, dev='png', dpi=300, fig.cap=c('Single Frame Showing Skew-normal Components', 'Single Frame Showing Beta Distribution for Type I Red')>>=
panelPlots(melon.raw, raw.I.red.mixes, raw.I.green.mixes,
           raw.II.mixes, plot='frames', frame.nums=c(1,3), norm='Raw')
@

%\begin{figure}[ht!]
%\centering
%\includegraphics[width=\linewidth]{singlePlots-1.png}
%\caption{single pane}
%\label{fig:int}
%\end{figure}

%\begin{figure}[ht!]
%\centering
%\includegraphics[width=\linewidth]{singlePlots-3.png}
%\caption{single pane}
%\label{fig:beta}
%\end{figure}

The graph includes the histogram of the intensities expressed as probabilities as would be produced by the generic \Rcode{hist} function. The dotted line represents the kernel density estimation. The colored curves are the probability distributions of the individual components with the vertical solid and dotted lines being the mode and mean, respectively, for each component. The legend identifies the mean and the mode and provides a floating point value, rounded off to three decimal places, for each. The solid black line is the sum of the components, representing the fit of the Skew-normal mixture model.

\clearpage
\section{sessionInfo}

<<sessionInfo, results='asis', echo=FALSE>>=
toLatex(sessionInfo())
@

\nocite{*}
\bibliographystyle{unsrturl}
\bibliography{skewr}

\end{document}

\name{getBarcodes}
\alias{getBarcodes}

\title{
Get barcodes from idat file names
}
\description{
A convenience function for retrieving simple barcodes from idat file names.
}
\usage{
getBarcodes(path = getwd(), recurse = FALSE)
}

\arguments{
  \item{path}{
The path or a character vector to the directory or directories in which to find the idat files.
}
  \item{recurse}{
  logical; should the function check subdirectories to derive barcodes from any found idat files. The default is \code{FALSE}.
}
}
\details{
Barcodes will be generated by all found idats in path(s). The default path is the current working directory.
}
\value{
A character vector of barcodes.
}

\author{
Ryan Putney \email{ryanputney@gmail.com}
}

\seealso{
\code{\link{getMethyLumiSet}}
}
\examples{

if(require(minfiData)){
  path <- system.file("extdata/5723646052", package="minfiData")
  barcodes <- getBarcodes(path = path)
}
}

\keyword{utilities}

\name{getSNparams}
\alias{getSNparams}

\title{
Estimate parameters for finite mixture of Skew-Normal distributions
}
\description{
Utilizes \code{smsn.mix} from the \code{mixsmsn} package to find the parameters for a finite mixture of skew normal distributions to model the overall distribution of signal intensities for a subset of probes on the Illumina Infinium HumanMethylation450. The probes may be subset by type and methylated or unmethylated. It can also be specified whether the SNP(rs), imprinted(idmr), or ch probes should be included or filtered out prior to parameter estimation.
}
\usage{
getSNparams(MethyLumiSet, allele = c('M', 'U'),
            type = c('I-red', 'I-green', 'II'),
            snps = TRUE, idmr = TRUE, ch = FALSE)
}

\arguments{
  \item{MethyLumiSet}{
A \code{MethyLumiSet} object
}
  \item{allele}{
Should parameter estimation be done on the methylated or unmethylated signal intensities
}
  \item{type}{
Use the signal intensities for which probe type
}
  \item{snps}{
logical; should the rs probes be included in the dataset. The default is \code{TRUE}
}
  \item{idmr}{
logical; should the probes of imprinted gene loci be included in the dataset. The default is \code{TRUE}
}
  \item{ch}{
logical; should the ch probes be included in the dataset. The default is \code{FALSE}
}

}

\value{
A \code{Skew.normal} object as returned by \code{smsn.mix} from the \code{mixsmsn} package with the means and modes of the components added.
}
\references{
Pidsley R, Wong CCY, Volta M, Lunnon K, Mill J, Schalwyk LC(2013). A data-driven approach to preprocessing Illumina 450k methylation array data. BMC Genomics, 14:293.

Prates MO, Cabral CRB, Lachos VH (2013).mixsmsn: Fitting Finite Mixture of Scale Mixture of Skew-Normal Distributions. Journal of Statistical Software, 54(12), 1-20. \url{http://www.jstatsoft.org/v54/i12/}
}
\author{

Ryan Putney \email{ryanputney@gmail.com}
}

\seealso{
\code{\link{subsetProbes}}
}
\examples{
\donttest{
if(require('wateRmelon')) {
  data(melon)
  mixes.raw.meth.II <- getSNparams(melon[,1], 'M', 'II')
}
}
}
\keyword{ models }

\name{panelPlots}
\alias{panelPlots}

\title{
Plot the distributions of the probe intensities and the components of the skew-normal mixture model
}
\description{
Creates a panel of nine plots. Six of the plots represent the density of either the methylated intensity or the unmethylated intensity given by one of three subsets of the 485,577 total probes. These subsets include Type I-red, Type I-green, and Type II.The remaining three distributions give the density of the beta-values for these same three subsets. Each of the nine plots optionally displays the distributions of the "rs" SNP probes and the probes associated with imprinted genes(Pidsley,2013) as a series of 'tick' marks located above the x-axis.
}
\usage{
panelPlots(MethyLumiSet, typeIRedModels, typeIGreenModels, typeIIModels,
           plot = c("panel", "frames"), samp.num = NULL, frame.nums = 1:9,
           norm = "", idmr = TRUE, snps = TRUE)
}

\arguments{
  \item{MethyLumiSet}{
The \code{MethyLumiSet} object from which the mixture models were derived
}
  \item{typeIRedModels}{
A \code{list} of the Type I-red mixture models listed in the following order: methylated models followed by unmethylated models
}
  \item{typeIGreenModels}{
A \code{list} of the Type I-green mixture models listed in the following order: methylated models followed by unmethylated models
}
  \item{typeIIModels}{
A \code{list} of the Type II mixture models listed in the following order: methylated models followed by unmethylated models
}
  \item{plot}{
Should the output consist of panel plots--one panel per sample or a single panel if \code{samp.num} is specified; or should the function output separate plots corresponding to the frames, given by \code{frame.nums}, for a single sample. The default is \code{"panel"}. If set to \code{"frames"}, \code{samp.num} must be specified
}
  \item{samp.num}{
If plotting for a single sample is desired, for which sample. The number given simply refers to the \code{MethyLumiSet} column that corresponds to the sample of interest
}
  \item{frame.nums}{
if \code{plot} is set to \code{"frame"}, then frame.nums is a vector that specifies which frames of the panel to plot. The default is to plot all nine frames. The frames are numbered from 1 to 9 in column-major order starting with the top left. For example, to plot the four corners, use \code{frame.nums=c(1,3,7,9)}
}
  \item{norm}{
A character string which will be displayed as part of the main title for each plot. Useful in indicated which normalization method was used for the modeled and plotted data
}
  \item{idmr}{
logical; should the intensities of the idmr probes be plotted as a series of tick-marks above the x-axis. The default is \code{TRUE}
}
  \item{snps}{
logical; should the intensities of the rs probes be plotted as a series of tick-marks above the x-axis. The default is \code{TRUE}
}
}

\value{
No return value. Only plots are generated.
}
\references{
Prates MO, Cabral CRB, Lachos VH (2013).mixsmsn: Fitting Finite Mixture of Scale Mixture of Skew-Normal Distributions. Journal of Statistical Software, 54(12), 1-20. \url{http://www.jstatsoft.org/v54/i12/}
}
\author{
Ryan Putney \email{ryanputney@gmail.com}
}
\note{
Please refer to the vignette for an example workflow.
}



\seealso{
\code{\link{getSNparams}}
}
\examples{
\donttest{
if(require('minfiData')) {
  path <- system.file("extdata/5723646052", package="minfiData")
  methylumiset.raw <- getMethyLumiSet(path = path)
  mixes.raw.meth.I.red <- getSNparams(methylumiset.raw, 'M', 'I-red')
  mixes.raw.meth.I.green <- getSNparams(methylumiset.raw, 'M', 'I-green')
  mixes.raw.meth.II <- getSNparams(methylumiset.raw, 'M', 'II')
  mixes.raw.unmeth.I.red <- getSNparams(methylumiset.raw, 'U', 'I-red')
  mixes.raw.unmeth.I.green <- getSNparams(methylumiset.raw, 'U', 'I-green')
  mixes.raw.unmeth.II <- getSNparams(methylumiset.raw, 'U', 'II')
  mixes.I.red <- list(mixes.raw.meth.I.red, mixes.raw.unmeth.I.red)
  mixes.I.green <- list(mixes.raw.meth.I.green, mixes.raw.unmeth.I.green)
  mixes.II <- list(mixes.raw.meth.II, mixes.raw.unmeth.II)
  panelPlots(methylumiset.raw, mixes.I.red, mixes.I.green, mixes.II)
}
}
}

\keyword{ aplot }

\name{getMethyLumiSet}
\alias{getMethyLumiSet}

\title{
Read idat files and create a \code{MethyLumiSet} object
}
\description{
This a wrapper function for \code{methylumIDAT} that does not require a vector of barcodes to be provided.
}
\usage{
getMethyLumiSet(path = getwd(), barcodes = NULL,
                norm = c("none", "illumina", "SWAN", "dasen"),
                bg.corr = TRUE)
}

\arguments{
  \item{path}{
The path to the directory containing the idat files.
}
  \item{barcodes}{
A vector of barcodes specifying which idat's to read.
}
  \item{norm}{
Should normalization be done on the resulting \code{MethyLumiSet}. The default is "none"
}
  \item{bg.corr}{
logical; if \code{TRUE}, an Illumina style background subtraction will be performed only if \code{norm} is set to \code{'illumina'}. Otherwise, it is ignored. If background subtraction without any normalization is desired, the \code{preprocess} method must be used.
}
}
\details{
If only \code{path} is provided, all idat's found in the given folder will be pulled. If only \code{barcodes} is given, corresponding idat's will be pulled from the current working directory. Both \code{path} and \code{barcodes} may be passed for finer control. The default is to pull all idat's found in the current working directory.
}
\value{
A \code{MethyLumiSet} object
}
\references{
Davis S, Du P, Bilke S, Triche T, Jr. and Bootwalla M (2014). methylumi: Handle Illumina methylation data. R package version 2.12.0.
}
\author{
Ryan Putney \email{ryanputney@gmail.com}
}
\note{
One would probably not normally want to use the preprocess option at this stage. It is more likely that a \code{MethyLumiSet} of the raw data will be desired. Then the \code{preprocess} method may be used to normalize the raw data or use background subtraction only on the raw data. See the vignette for example workflow.
}

\seealso{
\code{\link{getBarcodes}}, \code{\link{preprocess}}
}
\examples{
\donttest{
if(require('minfiData')) {
  path <- system.file("extdata/5723646052", package="minfiData")
  barcodes <- getBarcodes(path = path)
  methylumiset.raw <- getMethyLumiSet(path = path, barcodes = barcodes[1])
}
}
}
\keyword{ IO }

\name{subsetProbes}
\alias{subsetProbes}

\title{
Conveniently subset probes by type and retrieve the methylated or unmethylated intensities
}
\description{
Thus function accepts a \code{MethyLumiSet} object generated by \code{methylumi} or a \code{MethylSet} object generated by \code{minfi}. It will subset the probes by type--\code{"I-red"}, \code{"I-green"}, or \code{"II"}--and return a matrix of the methylated, \code{"M"}, or unmethylated, \code{"U"} signal intensities. It is also possible to include or filter out probes according to whether they are CpG sites(cg), SNPs(rs), imprinted(idmr) gene sites, or non-CpG loci(ch).
}
\usage{
subsetProbes(object, allele = c("M", "U"),
              type = c("I-red", "I-green", "II"),
              cg = TRUE, snps = TRUE, idmr = TRUE, ch = FALSE)
}

\arguments{
  \item{object}{
A \code{MethyLumiSet} or \code{MethylSet} object
}
  \item{allele}{
Should methylated or unmethylated data for the probes be returned.
}
  \item{type}{
May be \code{"I-red"}, \code{"I-green"}, or {"II"}.
}
  \item{cg}{
Logical; Should the returned dataset contain the CpG probes. The default is \code{TRUE}
}
  \item{snps}{
Logical; Should the returned dataset conain the rs probes. The default is \code{TRUE}
}
  \item{idmr}{
Logical; should the returned dataset include probes that interrogate imprinted gene sites as given by Pidsley et al.(2013). The default is \code{TRUE}
}
  \item{ch}{
Logical; should the returned dataset include the non-CpG (ch) probes. The default if \code{FALSE}
}
}

\value{
A \code{matrix}
}
\references{
Pidsley R, Wong CCY, Volta M, Lunnon K, Mill J, Schalwyk LC(2013). A data-driven approach to preprocessing Illumina 450k methylation array data. BMC Genomics, 14:293.
}
\author{
Ryan Putney \email{ryanputney@gmail.com}
}

\seealso{
\code{\link{getSNparams}}
}
\examples{
if(require('wateRmelon')) {
  data(melon)
  melon.meth.II <- subsetProbes(melon, 'M', 'II')
}
}

\keyword{ manip }

\name{preprocess}
\alias{preprocess}

\title{
Normalize a MethyLumiSet object using some popular choices
}
\description{
This is a wrapper function that allows normalizing of a MethyLumiSet using either a BeadStudio approximation, SWAN, or dasen. If desired, background correction only may be performed on the raw data.
}
\usage{
preprocess(MethyLumiSet, norm = c("none", "illumina", "SWAN", "dasen"),
           bg.corr = TRUE)
}

\arguments{
  \item{MethyLumiSet}{
A \code{MethyLumiSet} object
}
  \item{norm}{
The normalization method to be used
}
  \item{bg.corr}{
If TRUE, background subtarction using negative controls is performed. Ignored unless \code{norm} equals \code{'illumina'} or \code{'none'}
}
}
\details{
Both Illumina style normalization via controls and the background correct method are handled by \code{methylumi}. The \code{SWAN} and \code{dasen} normalization methods are both performed by \code{wateRmelon}
}
\value{
A \code{MethyLumiSet}
}
\references{
Davis S, Du P, Bilke S, Triche T, Jr. and Bootwalla M (2014). methylumi: Handle Illumina methylation data. R package version 2.12.0.

Maksimovic J, Gordon L, Oshlack A (2012). SWAN: Subset Quantile Within-Array Normalization for Illumina Infinium HumanMethylation450 BeadChips. Genome Biology, 13:R44.

Pidsley R, Wong CCY, Volta M, Lunnon K, Mill J, Schalwyk LC(2013). A data-driven approach to preprocessing Illumina 450k methylation array data. BMC Genomics, 14:293.

Schalkwyk LC, Pidsley R, Wong CC, Touleimat N, Defrance M, Teschendorff A and Maksimovic J (2013). wateRmelon: Illumina 450 methylation array normalization and metrics. R package version 1.5.1.


}
\author{
Ryan Putney \email{ryanputney@gmail.com}
}

\seealso{
\code{\link{getMethyLumiSet}}
}
\examples{
\donttest{
if(require('wateRmelon')) {
  data(melon)
  melon.dasen <- preprocess(melon, norm = 'dasen')
}
}
}
\keyword{ manip }
